@@ -23,7 +23,7 @@ class MainWidget(UiComponent):
         pass
 
     def show_information(self, text):
-        print text
+        print(text)
 
     def show_question(self, text, option0, option1, option2=""):
 
@@ -32,7 +32,10 @@ class MainWidget(UiComponent):
         raise NotImplementedError
 
     def show_error(self, text):
-        print text
+        print(text)
+
+    def handle_keyboard_interrupt(self, text):
+        self.show_error(text)
 
     def default_font_size(self):
         return 12
@@ -56,7 +59,7 @@ class MainWidget(UiComponent):
 
         """
 
-        print text
+        print(text)
 
     def set_progress_range(self, maximum):
 
@@ -102,5 +105,24 @@ class MainWidget(UiComponent):
     def enable_delete_current_card(self, is_enabled):
         pass
 
+    def enable_reset_current_card(self, is_enabled):
+        pass
+
     def enable_browse_cards(self, is_enabled):
         pass
+
+    # Moved here from default_controller.py to simplify the multithreaded
+    # implementation.
+    def show_export_metadata_dialog(self, metadata=None, read_only=False):
+        self.stopwatch().pause()
+        self.flush_sync_server()
+        dialog = self.component_manager.current("export_metadata_dialog")\
+            (component_manager=self.component_manager)
+        if metadata:
+            dialog.set_values(metadata)
+        if read_only:
+            dialog.set_read_only()
+        dialog.activate()
+        self.stopwatch().unpause()
+        return dialog.values()
+
