@@ -2,7 +2,7 @@
 # SM2_controller.py <Peter.Bienstman@UGent.be>
 #
 
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.libmnemosyne.review_controller import ReviewController
 
 ACQ_PHASE = 0
@@ -40,14 +40,7 @@ class SM2Controller(ReviewController):
     tooltip[RET_PHASE][5] = \
         _("Correct answer, but without any difficulties. The interval was probably too short.")
 
-    def reset(self):
-
-        """A Plugin can have a new scheduler, a new review controller or both,
-        and in order to avoid running this time consuming code more than once,
-        it is not folded into 'activate'.
-
-        """
-
+    def reset(self, new_only=False):
         self.card = None
         self._state = "EMPTY"
         self.learning_ahead = False
@@ -55,10 +48,8 @@ class SM2Controller(ReviewController):
         self.scheduled_count = None
         self.active_count = None
         self.rep_count = 0
-        self.widget = self.component_manager.current("review_widget")\
-                      (self.component_manager)
-        self.widget.activate()
-        self.scheduler().reset()
+        self.new_only = new_only
+        self.scheduler().reset(new_only=new_only)
         self.show_new_question()
 
     def reset_but_try_to_keep_current_card(self):
@@ -77,7 +68,7 @@ class SM2Controller(ReviewController):
         except Exception: # No previous card, or it was deleted.
             self.previous_card = None
         sch = self.scheduler()
-        sch.reset()
+        sch.reset(self.new_only)
         sch.rebuild_queue()
         self.reload_counters()
         self.update_status_bar_counters()
@@ -102,7 +93,7 @@ class SM2Controller(ReviewController):
         self._state = state
 
     def show_new_question(self):
-        self.widget.stop_media()
+        self.review_widget().stop_media()
         # Reload the counters if they have not yet been initialised. Also do
         # this if the active counter is zero, make sure it is really zero to
         # get a correct test for no more cards.
@@ -117,7 +108,8 @@ class SM2Controller(ReviewController):
                 self._state = "SELECT SHOW"
             else:
                 if self.config()["shown_schedule_help"] == False:
-                    self.main_widget().show_information(_("You have no more work for today. Either add more cards or come back tomorrow."))
+                    self.main_widget().show_information(\
+_("You have no more work for today. Either add more cards or come back tomorrow."))
                     self.config()["shown_schedule_help"] = True
                 self._state = "SELECT AHEAD"
         self.update_dialog()
@@ -126,7 +118,8 @@ class SM2Controller(ReviewController):
     def show_answer(self):
         if self._state == "SELECT AHEAD":
             if self.config()["warned_about_learning_ahead"] == False:
-                self.main_widget().show_information(_("Use 'Learn ahead of schedule' sparingly. For cramming before an exam, it's much better to use the cramming scheduler plugin"))
+                self.main_widget().show_information(\
+_("Use 'Learn ahead of schedule' sparingly. For cramming before an exam, it's much better to use the cramming scheduler plugin"))
                 self.config()["warned_about_learning_ahead"] = True
             self.learning_ahead = True
             self.show_new_question()
@@ -140,15 +133,18 @@ class SM2Controller(ReviewController):
         """Note that this also pulls in a new question."""
 
         self.flush_sync_server()
-        # Guide the learning process. 
+        if self.card is None:
+            return
+        # Guide the learning process.
         if self.config()["shown_learn_new_cards_help"] == False:
             if self.scheduled_count == 1:
-                self.main_widget().show_information(_("You have finished your scheduled reviews. Now, learn as many failed or new cards as you feel like."))
-                self.config()["shown_learn_new_cards_help"] = True          
+                self.main_widget().show_information(\
+_("You have finished your scheduled reviews. Now, learn as many failed or new cards as you feel like."))
+                self.config()["shown_learn_new_cards_help"] = True
         card_to_grade = self.card
         previous_grade = card_to_grade.grade
         self.update_counters(previous_grade, grade)
-        self.rep_count += 1     
+        self.rep_count += 1
         if self.scheduler().is_prefetch_allowed(card_to_grade):
             self.show_new_question()
             interval = self.scheduler().grade_answer(card_to_grade, grade)
@@ -201,18 +197,20 @@ class SM2Controller(ReviewController):
         self.update_grades_area()
         self.update_status_bar_counters()
         self.update_menu_bar()
-        self.widget.redraw_now()  # Don't wait until disk activity dies down.
+        # Don't wait until disk activity dies down.
+        self.review_widget().redraw_now()
 
     def update_qa_area(self, redraw_all=False):
         if redraw_all and self.card:
             self.card = \
                 self.database().card(self.card._id, is_id_internal=True)
-        w = self.widget
+        w = self.review_widget()
         # When the answer should be shown on top of the question, we draw the
         # answer in the question field to eliminate flicker.
         w.set_question_box_visible(True)
         if self.card is not None:
-            if self.card.fact_view.a_on_top_of_q:
+            if self.config()["QA_split"] == "single_window" or \
+               self.card.fact_view.a_on_top_of_q:
                 w.set_answer_box_visible(False)
             else:
                 w.set_answer_box_visible(True)
@@ -228,7 +226,8 @@ class SM2Controller(ReviewController):
             # Giving the widget info about the answer even before it is shown
             # allows it to optimise its layout.
             w.set_question(self.card.question(self.render_chain))
-            if not self.card.fact_view.a_on_top_of_q:
+            if self.config()["QA_split"] == "adaptive" and \
+               not self.card.fact_view.a_on_top_of_q:
                 w.set_answer(self.card.answer(\
                     self.render_chain, no_side_effects=True))
             w.reveal_question()
@@ -236,14 +235,19 @@ class SM2Controller(ReviewController):
         if self.card is None or self._state == "SELECT SHOW":
             w.clear_answer()
         else:
-            if not self.card.fact_view.a_on_top_of_q:
+            if not self.config()["QA_split"] == "single_window" and \
+               not self.card.fact_view.a_on_top_of_q:
                 w.set_answer(self.card.answer(\
                     self.render_chain, no_side_effects=False))
                 w.reveal_answer()
             else:
                 # Draw answer in question box.
-                w.set_question_label("Answer:")
-                w.set_question(self.card.answer(self.render_chain))
+                question_label_text = "Answer: "
+                if self.config()["show_tags_during_review"]:
+                    question_label_text += self.card.tag_string()
+                w.set_question_label(question_label_text)
+                w.set_question(self.card.answer(self.render_chain,
+                        a_on_top_of_q=self.card.fact_view.a_on_top_of_q))
                 w.reveal_question()
         # Update 'Show answer' button.
         if self._state == "EMPTY":
@@ -262,7 +266,7 @@ class SM2Controller(ReviewController):
         w.update_show_button(text, default, show_enabled)
 
     def update_grades_area(self):
-        w = self.widget
+        w = self.review_widget()
         # Update grade buttons.
         if self.card and self.card.grade < 2:
             phase = ACQ_PHASE
@@ -295,8 +299,8 @@ class SM2Controller(ReviewController):
             # Button text.
             if self._state == "SELECT GRADE" and \
                self.config()["show_intervals"] == "buttons":
-                w.set_grade_text(grade, str(self.scheduler().process_answer(\
-                                            self.card, grade, dry_run=True)))
+                w.set_grade_text(grade, self.scheduler().process_answer(\
+                                        self.card, grade, dry_run=True))
             else:
                 w.set_grade_text(grade, str(grade))
 
@@ -313,13 +317,16 @@ class SM2Controller(ReviewController):
             else:
                 w.enable_edit_current_card(False)
         w.enable_delete_current_card(self.card != None)
+        w.enable_reset_current_card(self.card != None)
         w.enable_browse_cards(self.database().is_loaded())
 
     def update_status_bar_counters(self):
-        self.widget.update_status_bar_counters()
+        self.review_widget().update_status_bar_counters()
 
     def is_question_showing(self):
-        return self.review_controller()._state == "SELECT SHOW"
+        return self.review_controller()._state == "SELECT SHOW" \
+               and self.card is not None
 
     def is_answer_showing(self):
-        return self.review_controller()._state == "SELECT GRADE"
+        return self.review_controller()._state == "SELECT GRADE" \
+               and self.card is not None
