@@ -4,14 +4,14 @@
 
 import time
 import random
-import calendar
 import datetime
 
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.libmnemosyne.scheduler import Scheduler
 
-HOUR = 60 * 60 # Seconds in an hour.
-DAY = 24 * HOUR # Seconds in a day.
+
+HOUR = 60 * 60  # Seconds in an hour.
+DAY = 24 * HOUR  # Seconds in a day.
 
 
 class SM2Mnemosyne(Scheduler):
@@ -33,60 +33,7 @@ class SM2Mnemosyne(Scheduler):
     """
 
     name = "SM2 Mnemosyne"
-
-    def midnight_UTC(self, timestamp):
-
-        """Round a timestamp to a value with resolution of a day, storing it
-        in a timezone independent way, as a POSIX timestamp corresponding to
-        midnight UTC on that date.
-
-        E.g. if the scheduler sets 'next_rep' to 2012/1/1 12:14 local time,
-        this function will return the timestamp corresponding to
-        2012/1/1 00;00 UTC.
-
-        Important: the timestamp needs to have the meaning of local time,
-        not e.g. UTC, so calling midnight_UTC twice will give unexpected
-        results.
-
-        """
-
-        # Create a time tuple containing the local date only, i.e. throwing
-        # away hours, minutes, etc.
-        date_only = datetime.date.fromtimestamp(timestamp).timetuple()
-        # Now we reinterpret this same time tuple as being UTC and convert it
-        # to a POSIX timestamp. (Note that timetuples are 'naive', i.e. they
-        # themselves do not contain timezone information.)
-        return int(calendar.timegm(date_only))
-
-    def adjusted_now(self, now=None):
-
-        """Timezone information and 'day_starts_at' will only become relevant
-        when the queue is built, not at schedule time, to allow for
-        moving to a different timezone after a card has been scheduled.
-        Cards are due when 'adjusted_now >= next_rep', and this function
-        makes sure that happens at h:00 local time (with h being
-        'day_starts_at').
-
-        """
-
-        if now == None:
-            now = time.time()
-        # The larger 'day_starts_at', the later the card should become due,
-        # i.e. larger than 'next_card', so the more 'now' should be decreased.
-        now -= self.config()["day_starts_at"] * HOUR
-        # 'altzone' or 'timezone' contains the offset in seconds west of UTC.
-        # This number is positive for the US, where a card should become
-        # due later than in Europe, so 'now' should be decreased by this
-        # offset.
-        # As for when to use 'altzone' instead of 'timezone' if daylight
-        # savings time is active, this is a matter of big confusion
-        # among the Python developers themselves:
-        # http://bugs.python.org/issue7229
-        if time.localtime(now).tm_isdst and time.daylight:
-            now -= time.altzone
-        else:
-            now -= time.timezone
-        return int(now)
+    _warned_about_too_many_cards = False
 
     def true_scheduled_interval(self, card):
 
@@ -102,9 +49,9 @@ class SM2Mnemosyne(Scheduler):
 
         interval = card.next_rep - card.last_rep
         if card.grade < 2:
-            if interval != 0:
-                self.main_widget().show_error("Internal error: interval not zero.")
-            #assert interval == 0
+            if abs(interval) > 1e-10:
+                self.main_widget().show_error(\
+                    "Internal error: interval not zero.")
             return interval
         interval += self.config()["day_starts_at"] * HOUR
         if time.localtime(time.time()).tm_isdst and time.daylight:
@@ -113,7 +60,7 @@ class SM2Mnemosyne(Scheduler):
             interval += time.timezone
         return int(interval)
 
-    def reset(self):
+    def reset(self, new_only=False):
 
         """'_card_ids_in_queue' contains the _ids of the cards making up the
         queue.
@@ -138,10 +85,13 @@ class SM2Mnemosyne(Scheduler):
 
         self._card_ids_in_queue = []
         self._fact_ids_in_queue = []
-        self._fact_ids_memorised = []
+        self._fact_ids_memorised = self._fact_ids_learned_today()
         self._card_id_last = None
-        self.stage = 1
-        self.warned_about_too_many_cards = False     
+        self.new_only = new_only
+        if self.new_only == False:
+            self.stage = 1
+        else:
+            self.stage = 3
 
     def set_initial_grade(self, cards, grade):
 
@@ -208,6 +158,7 @@ class SM2Mnemosyne(Scheduler):
             return
         self._card_ids_in_queue = []
         self._fact_ids_in_queue = []
+        self._warned_about_too_many_cards = self._already_warned_today()
 
         # Stage 1
         #
@@ -220,14 +171,15 @@ class SM2Mnemosyne(Scheduler):
         if self.stage == 1:
             if self.config()["shown_backlog_help"] == False:
                 if db.scheduled_count(self.adjusted_now() - DAY) != 0:
-                    self.main_widget().show_information(_("You appear to have missed some reviews. Don't worry too much about this backlog, and do as many cards as you feel comfortable with to catch up each day. Mnemosyne will automatically reschedule your cards such that the most urgent ones are shown first."))
+                    self.main_widget().show_information(\
+_("You appear to have missed some reviews. Don't worry too much about this backlog, and do as many cards as you feel comfortable with to catch up each day. Mnemosyne will automatically reschedule your cards such that the most urgent ones are shown first."))
                     self.config()["shown_backlog_help"] = True
             if self.config()["randomise_scheduled_cards"] == True:
                 sort_key = "random"
             else:
                 sort_key = "interval"
             for _card_id, _fact_id in db.cards_due_for_ret_rep(\
-                    self.adjusted_now(), sort_key=sort_key, limit=50):
+                self.adjusted_now(), sort_key=sort_key, limit=50):
                 self._card_ids_in_queue.append(_card_id)
                 self._fact_ids_in_queue.append(_fact_id)
             if len(self._card_ids_in_queue):
@@ -323,10 +275,13 @@ class SM2Mnemosyne(Scheduler):
                     self._fact_ids_in_queue.append(_fact_id)
                     non_memorised_in_queue += 1
                     if non_memorised_in_queue == limit:
-                        self.stage = 2
+                        if self.new_only == False:
+                            self.stage = 2
+                        else:
+                            self.stage = 3
                         return
-            # If the queue is close to empty, start pulling in sister cards.
-            if len(self._fact_ids_in_queue) <= 2:
+            # If our hand is not full enough, start pulling in sister cards.
+            if non_memorised_in_queue < limit:
                 for _card_id, _fact_id in db.cards_unseen(\
                         sort_key=sort_key, limit=min(limit, 50)):
                     if _fact_id not in self._fact_ids_in_queue:
@@ -334,7 +289,10 @@ class SM2Mnemosyne(Scheduler):
                         self._fact_ids_in_queue.append(_fact_id)
                         non_memorised_in_queue += 1
                         if non_memorised_in_queue == limit:
-                            self.stage = 2
+                            if self.new_only == False:
+                                self.stage = 2
+                            else:
+                                self.stage = 3
                             return
             # If the queue is still empty, go to learn ahead of schedule.
             if len(self._card_ids_in_queue) == 0:
@@ -349,10 +307,13 @@ class SM2Mnemosyne(Scheduler):
         # earliest scheduled cards first. We only put 50 cards at the same
         # time into the queue, in order to save memory.
         if learn_ahead == False:
-            self.stage = 2
+            if self.new_only == False:
+                self.stage = 2
+            else:
+                self.stage = 3
             return
         for _card_id, _fact_id in db.cards_learn_ahead(self.adjusted_now(),
-                sort_key="next_rep", limit=50):
+            sort_key="next_rep", limit=50):
             self._card_ids_in_queue.append(_card_id)
         # Relearn cards which we got wrong during learn ahead.
         self.stage = 2
@@ -526,6 +487,10 @@ class SM2Mnemosyne(Scheduler):
         # Allow plugins to modify new_interval by multiplying it.
         new_interval *= self.interval_multiplication_factor(card, new_interval)
         new_interval = int(new_interval)
+        # Optional: limit interval:
+        if self.config()["max_scheduled_interval_days"]:
+            if new_interval > self.config()["max_scheduled_interval_days"] * DAY:
+                new_interval = self.config()["max_scheduled_interval_days"] * DAY    
         # When doing a dry run, stop here and return the scheduled interval.
         if dry_run:
             return new_interval
@@ -541,12 +506,8 @@ class SM2Mnemosyne(Scheduler):
         else:
             card.next_rep = card.last_rep
         # Warn if we learned a lot of new cards.
-        if len(self._fact_ids_memorised) == 15 and \
-            self.warned_about_too_many_cards == False:
-            self.main_widget().show_information(\
-        _("You've memorised 15 new or failed cards.") + " " +\
-        _("If you do this for many days, you could get a big workload later."))
-            self.warned_about_too_many_cards = True       
+
+        self._warn_too_many_cards()
         # Run hooks.
         self.database().current_criterion().apply_to_card(card)
         for f in self.component_manager.all("hook", "after_repetition"):
@@ -580,72 +541,56 @@ class SM2Mnemosyne(Scheduler):
         else:
             return self.database().card_count_scheduled_n_days_ago(-n)
 
-    def next_rep_to_interval_string(self, next_rep, now=None):
+    def _fact_ids_learned_today(self):
+        """It loads the learned _fact_ids back from the logs in order not
+        to forget the learned cards when the app is closed and re-opened.
 
-        """Converts next_rep to a string like 'tomorrow', 'in 2 weeks', ...
+        """
+        db = self.database()
+        if not db.is_loaded():
+            return []
+
+        start_of_day, end_of_day = self._today_start_and_end_timestamp()
+
+        forgotten_fact_ids = [_fact_id for _fact_id in db.fact_ids_forgotten_and_learned_today(start_of_day, end_of_day)]
+        new_fact_ids = [_fact_id for _fact_id in db.fact_ids_newly_learned_today(start_of_day, end_of_day)]
+
+        return new_fact_ids + forgotten_fact_ids
+
+    def _warn_too_many_cards(self):
+        """Shows a warning if there are already 15 new or failed cards memorized.
+
+        """
+        # only alert if it is exactly 15, do be obtrusive
+        if (len(self._fact_ids_memorised) == 15 and
+                not self._warned_about_too_many_cards):
+            self.main_widget().show_information(
+                ("You've memorised 15 new or failed cards.") + " " +
+                ("If you do this for many days, you could get a big workload later."))
+            self._warned_about_too_many_cards = True
+            # log the event, so we won't show an alert more than once a day
+            self.log().warn_too_many_cards()
+
+    def _today_start_and_end_timestamp(self):
+        timestamp = time.time() - 0 - self.config()["day_starts_at"] * HOUR
+        date_only = datetime.date.fromtimestamp(timestamp)  # Local date.
+        start_of_day = int(time.mktime(date_only.timetuple()))
+        start_of_day += self.config()["day_starts_at"] * HOUR
+        return start_of_day, start_of_day + DAY
+
+    def _already_warned_today(self):
+        """From the current session or from the database it checks if
+        there was a warning about learning too many cards or not.
+
+        If it is already set in the current session, return with that,
+        otherwise query the database (log table) for the warn event for
+        today.
 
         """
 
-        if now is None:
-            now = self.adjusted_now()
-        interval_days = (next_rep - now) / DAY
-        if interval_days >= 365:
-            interval_years = interval_days/365.
-            return _("in") + " " + "%.1f" % interval_years + " " + \
-                   _("years")
-        elif interval_days >= 62:
-            interval_months = int(interval_days/31)
-            return _("in") + " " + str(interval_months) + " " + \
-                   _("months")
-        elif interval_days >= 31:
-            return _("in 1 month")
-        elif interval_days >= 1:
-            return _("in") + " " + str(int(interval_days) + 1) + " " + \
-                   _("days")
-        elif interval_days >= 0:
-            return _("tomorrow")
-        elif interval_days >= -1:
-            return _("today")
-        elif interval_days >= -2:
-            return _("1 day overdue")
-        elif interval_days >= -31:
-            return str(int(-interval_days) - 1) + " " + _("days overdue")
-        elif interval_days >= -62:
-            return _("1 month overdue")
-        elif interval_days >= -365:
-            interval_months = int(-interval_days/31)
-            return str(interval_months) + " " + _("months overdue")
-        else:
-            interval_years = -interval_days/365.
-            return "%.1f " % interval_years +  _("years overdue")
+        if self._warned_about_too_many_cards:
+            return True
 
-    def last_rep_to_interval_string(self, last_rep, now=None):
-
-        """Converts next_rep to a string like 'yesterday', '2 weeks ago', ...
-
-        """
-
-        if now is None:
-            now = time.time()
-        # To perform the calculation, we need to 'snap' the two timestamps
-        # to midnight UTC before calculating the interval.
-        now = self.midnight_UTC(\
-            now - self.config()["day_starts_at"] * HOUR)
-        last_rep = self.midnight_UTC(\
-            last_rep - self.config()["day_starts_at"] * HOUR)
-        interval_days = (last_rep - now) / DAY
-        if interval_days > -1:
-            return _("today")
-        elif interval_days > -2:
-            return _("yesterday")
-        elif interval_days > -31:
-            return str(int(-interval_days)) + " " + _("days ago")
-        elif interval_days > -62:
-            return _("1 month ago")
-        elif interval_days > -365:
-            interval_months = int(-interval_days/31.)
-            return str(interval_months) + " " + _("months ago")
-        else:
-            interval_years = -interval_days/365.
-            return "%.1f " % interval_years +  _("years ago")
+        start_of_day, end_of_day = self._today_start_and_end_timestamp()
 
+        return self.database().has_already_warned_today(start_of_day, end_of_day)
