@@ -2,23 +2,23 @@
 # card_type_wdgt_generic.py <Peter.Bienstman@UGent.be>
 #
 
-from PyQt4 import QtCore, QtGui
+import re
 
-from mnemosyne.libmnemosyne.translator import _
+from PyQt5 import QtCore, QtGui, QtWidgets
+
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.pyqt_ui.qtextedit2 import QTextEdit2
 from mnemosyne.libmnemosyne.ui_components.card_type_widget \
      import GenericCardTypeWidget
 
 
-class GenericCardTypeWdgt(QtGui.QWidget, GenericCardTypeWidget):
+class GenericCardTypeWdgt(QtWidgets.QWidget, GenericCardTypeWidget):
 
-    def __init__(self, component_manager, parent, card_type):
-        QtGui.QWidget.__init__(self, parent)
-        GenericCardTypeWidget.__init__(self, component_manager)
-        self.card_type = card_type
-        self.hboxlayout = QtGui.QHBoxLayout(self)
-        self.hboxlayout.setMargin(0)
-        self.vboxlayout = QtGui.QVBoxLayout()
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
+        self.hboxlayout = QtWidgets.QHBoxLayout(self)
+        self.hboxlayout.setContentsMargins(0, 0, 0, 0)
+        self.vboxlayout = QtWidgets.QVBoxLayout()
         self.fact_key_for_edit_box = {}
         self.top_edit_box = None
         self.edit_boxes = []
@@ -31,14 +31,15 @@ class GenericCardTypeWdgt(QtGui.QWidget, GenericCardTypeWidget):
             pronunciation_hiding = self.config().card_type_property(\
                 "hide_pronunciation_field", self.card_type, default=False)
         # Construct the rest of the dialog.
+        parent = kwds["parent"] # Also used by other parent classes inits.
         parent.setTabOrder(parent.card_types_widget, parent.tags)
         for fact_key, fact_key_name in self.card_type.fact_keys_and_names:
-            l = QtGui.QLabel(_(fact_key_name) + ":", self)
+            l = QtWidgets.QLabel(_(fact_key_name) + ":", self)
             self.vboxlayout.addWidget(l)
             if fact_key == "p_1":
                 self.pronunciation_label = l
                 self.pronunciation_label.setVisible(not pronunciation_hiding)
-            t = QTextEdit2(self, pronunciation_hiding)
+            t = QTextEdit2(self, self.card_type, pronunciation_hiding)
             self.edit_boxes.append(t)
             t.setTabChangesFocus(True)
             t.setUndoRedoEnabled(True)
@@ -122,7 +123,7 @@ class GenericCardTypeWdgt(QtGui.QWidget, GenericCardTypeWidget):
 
     def is_empty(self):
         for edit_box in self.fact_key_for_edit_box:
-            if unicode(edit_box.document().toPlainText()):
+            if edit_box.document().toPlainText():
                 return False
         return True
 
@@ -131,14 +132,27 @@ class GenericCardTypeWdgt(QtGui.QWidget, GenericCardTypeWidget):
 
     def fact_data(self):
         _fact_data = {}
-        for edit_box, fact_key in self.fact_key_for_edit_box.iteritems():
-            _fact_data[fact_key] = unicode(edit_box.document().toPlainText())
+        for edit_box, fact_key in self.fact_key_for_edit_box.items():
+            _fact_data[fact_key] = edit_box.document().toPlainText()
         return _fact_data
 
+    def foreign_text(self):
+        foreign_fact_key = self.config().card_type_property(\
+            "foreign_fact_key", self.card_type, default=None)
+        if not foreign_fact_key:
+            return
+        foreign_text = self.fact_data()[foreign_fact_key]
+        # Strip Cloze brackets and hints.
+        if getattr(self.card_type, "q_a_from_cloze", None):
+            foreign_text = self.card_type.q_a_from_cloze(foreign_text, -1)[0]
+        # Strip html tags.
+        tag_re = re.compile(r"<[^>]+>")
+        return tag_re.sub("", foreign_text).strip()
+
     def set_fact_data(self, fact_data):
         self.fact_data_before_edit = fact_data
         if fact_data:
-            for edit_box, fact_key in self.fact_key_for_edit_box.iteritems():
+            for edit_box, fact_key in self.fact_key_for_edit_box.items():
                 if fact_key in fact_data:
                     edit_box.setPlainText(fact_data[fact_key])
 
