@@ -2,18 +2,18 @@
 # convert_card_type_keys_dlg.py <Peter.Bienstman@UGent.be>
 #
 
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtGui, QtWidgets
 
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.pyqt_ui.ui_convert_card_type_keys_dlg import \
      Ui_ConvertCardTypeKeysDlg
 
 
-class ConvertCardTypeKeysDlg(QtGui.QDialog, Ui_ConvertCardTypeKeysDlg):
+class ConvertCardTypeKeysDlg(QtWidgets.QDialog, Ui_ConvertCardTypeKeysDlg):
 
     def __init__(self, old_card_type, new_card_type, correspondence,
-                 check_required_fact_keys=True, parent=None):
-        QtGui.QDialog.__init__(self, parent)
+                 check_required_fact_keys=True, **kwds):
+        super().__init__(**kwds)
         self.setupUi(self)
         self.setWindowFlags(self.windowFlags() \
             | QtCore.Qt.WindowMinMaxButtonsHint)
@@ -27,14 +27,14 @@ class ConvertCardTypeKeysDlg(QtGui.QDialog, Ui_ConvertCardTypeKeysDlg):
         index = 1
         for old_fact_key, old_fact_key_name in \
             old_card_type.fact_keys_and_names:
-            label = QtGui.QLabel(self)
+            label = QtWidgets.QLabel(self)
             label.setText(_(old_fact_key_name) + ":")
             font = QtGui.QFont()
             font.setWeight(50)
             font.setBold(False)
             label.setFont(font)
             self.gridLayout.addWidget(label, index, 0, 1, 1)
-            combobox = QtGui.QComboBox(self)
+            combobox = QtWidgets.QComboBox(self)
             for new_fact_key, new_key_name in \
                 new_card_type.fact_keys_and_names:
                 combobox.addItem(_(new_key_name))
@@ -51,15 +51,14 @@ class ConvertCardTypeKeysDlg(QtGui.QDialog, Ui_ConvertCardTypeKeysDlg):
         for old_fact_key, old_fact_key_name in \
             self.old_card_type.fact_keys_and_names:
             new_fact_key_name = \
-                unicode(self.comboboxes[old_fact_key].currentText())
+                self.comboboxes[old_fact_key].currentText()
             if new_fact_key_name != _("<none>"):
                 self.ok_button.setEnabled(True)
                 new_fact_key = \
                      self.new_card_type.fact_key_with_name(new_fact_key_name)
                 if new_fact_key in self.correspondence.values():
-                    QtGui.QMessageBox.critical(self, _("Mnemosyne"),
-                        _("No duplicate in new fact keys allowed."),
-                        _("&OK"), "", "", 0, -1)
+                    QtWidgets.QMessageBox.critical(self, _("Mnemosyne"),
+                        _("No duplicate in new fact keys allowed."))
                     self.ok_button.setEnabled(False)
                     return
                 self.correspondence[old_fact_key] = new_fact_key
@@ -69,9 +68,8 @@ class ConvertCardTypeKeysDlg(QtGui.QDialog, Ui_ConvertCardTypeKeysDlg):
                     self.ok_button.setEnabled(False)
                     if len(self.correspondence) == \
                        len(self.old_card_type.fact_keys()):
-                        QtGui.QMessageBox.critical(self, _("Mnemosyne"),
-                            _("A required field is missing."),
-                            _("&OK"), "", "", 0, -1)
+                        QtWidgets.QMessageBox.critical(self, _("Mnemosyne"),
+                            _("A required field is missing."))
                     return
 
     def accept(self):
@@ -79,9 +77,9 @@ class ConvertCardTypeKeysDlg(QtGui.QDialog, Ui_ConvertCardTypeKeysDlg):
         for old_fact_key, old_fact_key_name in \
             self.old_card_type.fact_keys_and_names:
             new_fact_key_name = \
-                unicode(self.comboboxes[old_fact_key].currentText())
+                self.comboboxes[old_fact_key].currentText()
             if new_fact_key_name != _("<none>"):
                 new_fact_key = \
                      self.new_card_type.fact_key_with_name(new_fact_key_name)
                 self.correspondence[old_fact_key] = new_fact_key
-        QtGui.QDialog.accept(self)
+        QtWidgets.QDialog.accept(self)
