@@ -2,19 +2,18 @@
 # export_metadata_dlg.py <Peter.Bienstman@UGent.be>
 #
 
-from PyQt4 import QtGui, QtCore
+from PyQt5 import QtGui, QtCore, QtWidgets
 
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.pyqt_ui.ui_export_metadata_dlg import Ui_ExportMetadataDlg
 from mnemosyne.libmnemosyne.ui_components.dialogs import ExportMetadataDialog
 
 
-class ExportMetadataDlg(QtGui.QDialog, Ui_ExportMetadataDlg,
-    ExportMetadataDialog):
+class ExportMetadataDlg(QtWidgets.QDialog, ExportMetadataDialog,
+                        Ui_ExportMetadataDlg):
 
-    def __init__(self, component_manager):
-        ExportMetadataDialog.__init__(self, component_manager)
-        QtGui.QDialog.__init__(self, self.main_widget())
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
         self.setupUi(self)
         self.setWindowFlags(QtCore.Qt.Dialog \
                 | QtCore.Qt.CustomizeWindowHint \
@@ -47,7 +46,7 @@ class ExportMetadataDlg(QtGui.QDialog, Ui_ExportMetadataDlg,
         if "revision" in metadata:
             self.revision.setValue(int(metadata["revision"]))
         if "notes" in metadata:
-            self.notes.setPlainText(QtCore.QString.fromUtf8(metadata["notes"]))
+            self.notes.setPlainText(metadata["notes"])
 
     def set_read_only(self):
         self.card_set_name.setReadOnly(True)
@@ -79,23 +78,23 @@ class ExportMetadataDlg(QtGui.QDialog, Ui_ExportMetadataDlg,
             else:
                 event.ignore()
         else:
-            QtGui.QDialog.keyPressEvent(self, event)
+            QtWidgets.QDialog.keyPressEvent(self, event)
 
     def reject(self):
         self.cancelled = True
-        return QtGui.QDialog.reject(self)
+        return QtWidgets.QDialog.reject(self)
 
     def values(self):
         if self.cancelled:
             return None
         metadata = {}
-        metadata["card_set_name"] = unicode(self.card_set_name.text())
-        metadata["author_name"] = unicode(self.author_name.text())
-        metadata["author_email"] = unicode(self.author_email.text())
-        metadata["tags"] = unicode(self.tags.text())
-        metadata["date"] = unicode(self.date.date().toString())
+        metadata["card_set_name"] = self.card_set_name.text()
+        metadata["author_name"] = self.author_name.text()
+        metadata["author_email"] = self.author_email.text()
+        metadata["tags"] = self.tags.text()
+        metadata["date"] = self.date.date().toString()
         metadata["revision"] = str(self.revision.value())
-        metadata["notes"] = unicode(self.notes.toPlainText())
-        self.config()["author_name"] = unicode(metadata["author_name"])
-        self.config()["author_email"] = unicode(metadata["author_email"])
+        metadata["notes"] = self.notes.toPlainText()
+        self.config()["author_name"] = metadata["author_name"]
+        self.config()["author_email"] = metadata["author_email"]
         return metadata
\ No newline at end of file
