@@ -2,13 +2,13 @@
 # qpushbutton2.py <Peter.Bienstman@UGent.be>
 #
 
-from PyQt4 import QtGui
+from PyQt5 import QtGui, QtWidgets
 
 
-class QPushButton2(QtGui.QPushButton):
+class QPushButton2(QtWidgets.QPushButton):
 
     """QPushButton which throws away key repeats."""
 
     def keyPressEvent(self, event):
         if not event.isAutoRepeat():
-            return QtGui.QPushButton.keyPressEvent(self, event)
+            return QtWidgets.QPushButton.keyPressEvent(self, event)
