@@ -9,7 +9,7 @@ import socket
 from PyQt5 import QtCore
 
 from mnemosyne.libmnemosyne import Mnemosyne
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.libmnemosyne.component import Component
 from mnemosyne.libmnemosyne.utils import traceback_string
 from mnemosyne.libmnemosyne.sync_server import SyncServer
@@ -66,6 +66,8 @@ class ServerThread(QtCore.QThread, SyncServer):
     def __init__(self, **kwds):
         super().__init__(ui=self, **kwds)
         self.server_has_connection = False
+        # A fast moving progress bar seems to cause crashes on Windows.
+        self.show_numeric_progress_bar = (sys.platform != "win32")
 
     def run(self):
         try:
@@ -160,16 +162,20 @@ class ServerThread(QtCore.QThread, SyncServer):
         self.set_progress_text_signal.emit(text)
 
     def set_progress_range(self, maximum):
-        self.set_progress_range_signal.emit(maximum)
+        if self.show_numeric_progress_bar:
+            self.set_progress_range_signal.emit(maximum)
 
     def set_progress_update_interval(self, value):
-        self.set_progress_update_interval_signal.emit(value)
+        if self.show_numeric_progress_bar:
+            self.set_progress_update_interval_signal.emit(value)
 
     def increase_progress(self, value):
-        self.increase_progress_signal.emit(value)
+        if self.show_numeric_progress_bar:
+            self.increase_progress_signal.emit(value)
 
     def set_progress_value(self, value):
-        self.set_progress_value_signal.emit(value)
+        if self.show_numeric_progress_bar:
+            self.set_progress_value_signal.emit(value)
 
     def close_progress(self):
         self.close_progress_signal.emit()
